library(MLmetrics)   # caret requirement
library(MLeval)      # plot ROC curves from caret
library(gbm)         # caret requirement for Stochastic Gradient Boosting models
library(ranger)      # caret requirement for random forest models
library(rattle)      # visualizing decision trees
library(caret)       # interface to ML algorithms
library(pROC)
library(C50)         # contains the churn dataset

set.seed(12L)
head(iris)
# do a 3:4 train test split
index <- createDataPartition(iris$Species,p=.75, list=F)
train.set = iris[index,]
test.set = iris[-index,]

fitControl <- trainControl(## K-fold CV
  method = "repeatedcv",
  # K=10 in K-fold cross-validation
  number = 10,
  # repeated ten times
  repeats = 10,
  classProbs = T,
  savePredictions = T)
?train
# fit a decision tree model
iris.tree = train(Species ~ ., 
                  data=train.set, 
                  method="rpart", 
                  trControl = fitControl) # using cross validation
iris.tree
summary(iris.tree$finalModel)

plot(iris.tree$finalModel, uniform=TRUE,
     main="Classification Tree")
text(iris.tree$finalModel, cex=.8)

fancyRpartPlot(iris.tree$finalModel)

# estimate model quality
iris.pred = predict(iris.tree, newdata = test.set)
table(iris.pred, test.set$Species)

error.rate = round(mean(iris.pred != test.set$Species),2)
error.rate

confusionMatrix(iris.pred, test.set$Species)
print(iris.pred)
#

iris.gbm = train(Species ~ ., 
                 data=train.set, 
                 method="gbm", 
                 trControl = fitControl,
                 # preProcess = c("center","scale"),
                 verbose=F) 
iris.gbm
iris.pred = predict(iris.gbm, newdata = test.set)
error.rate = round(mean(iris.pred != test.set$Species),2)
error.rate
#

data(mlc_churn, package = "modeldata")
index <- createDataPartition(mlc_churn$churn,p=.67, list=F)
churnTrain <- mlc_churn[index, ]
churnTest <- mlc_churn[-index, ]
table(churnTrain$churn)/nrow(churnTrain)
#

# we want to make folds that can be re-used
myFolds <- createFolds(churnTrain$churn, k = 5)
sapply(myFolds, function(i) {
  table(churnTrain$churn[i])/length(i)
})
#
# pass the folds as index
fitControl <- trainControl(method="cv",
  summaryFunction = twoClassSummary,
  savePredictions = T,
  classProbs = T,
  index = myFolds)
#

dt_model = train(churn ~ .,
                 churnTrain,
                 metric = "ROC",
                 method="rpart", 
                 trControl = fitControl)
gbm_model = train(churn ~ .,
                  churnTrain,
                  metric = "ROC",
                  method="gbm", 
                  trControl = fitControl,
                  verbose=F) 
rf_model <- train(churn ~ .,
                  churnTrain,
                  metric = "ROC",
                  method = "rf",
                  trControl = fitControl)
rg_model <- train(churn ~ .,
                  churnTrain,
                  metric = "ROC",
                  method = "ranger",
                  trControl = fitControl)
knn_model <- train(churn ~ .,
                   churnTrain,
                   metric = "ROC",
                   method = "knn",
                   tuneLength = 20,
                   trControl = fitControl)

model_list <- list(gbm = gbm_model,
                   rf = rf_model,
                   rg = rg_model,
                   knn = knn_model,
                   dt = dt_model)
resamp <- resamples(model_list)
lattice::bwplot(resamp, metric = "ROC")
#

knn_model_prep <- train(churn ~ .,
                   churnTrain,
                   metric = "ROC",
                   method = "knn",
                   tuneLength = 20,
                   preProcess = c("center","scale"), # knn now with preprocessing
                   trControl = fitControl)

model_list <- list(gbm = gbm_model,
                   rf = rf_model,
                   knn = knn_model,
                   knn_prep = knn_model_prep,
                   dt = dt_model)
resamp <- resamples(model_list)
lattice::bwplot(resamp, metric = "ROC")

res_comp <- evalm(list(rf_model, knn_model_prep))
p <- predict(rf_model, newdata = churnTest)
confusionMatrix(p, churnTest$churn)
